package com.cours.test;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.cours.dao.IPersonneDao;
import com.cours.entities.Personne;
import com.cours.service.IServiceFacade;

public class JUnitPersonneDao {
	private static final Log log = LogFactory.getLog(JUnitPersonneDao.class);
	private static IPersonneDao personneDao = null;
	private static List<Personne> personnes = null;
	// Compter le nombre de personnes dans votre base de données.
	private static final int NB_PERSONNES_LIST = 14;
	private static final int NB_PERSONNES_FIND_BY_PRENOM = 2;
	private static final int NB_PERSONNES_FIND_BY_NOM = 2;
	private static final int NB_PERSONNES_FIND_BY_CODE_POSTAL = 6;
	private static final String FIND_PERSONNES_BY_PRENOM = "Julien";
	private static final String FIND_PERSONNES_BY_NOM = "Claire";
	private static final int NB_PERSONNES_FIND_BY_VILLE = 6;
	private static final String FIND_PERSONNES_BY_VILLE = "Paris";
	private static final String FIND_PERSONNES_BY_CODE_POSTAL = "75000";
	private static ClassPathXmlApplicationContext context = null;
	private static final String contextFileName = "applicationContext.xml";
	private static final String serviceBeanName = "serviceFacade";
	private static IServiceFacade serviceFacade = null;

	@BeforeClass
	public static void init() throws Exception {
		// configuration de l'application
		log.debug("Entree de la methode");
		context = new ClassPathXmlApplicationContext(contextFileName);
		serviceFacade = (IServiceFacade) context.getBean(serviceBeanName);
		personneDao = serviceFacade.getPersonneDao();
		personnes = personneDao.findAll();
		log.debug("Sortie de la methode");
	}

	public void verifyDataPersonnes(List<Personne> personnes) {
		log.debug("Entree de la methode");
		if (personnes != null) {
			log.debug("personnes.size(): " + personnes.size());
			for (Personne person : personnes) {
				verifyDataPersonne(person);
			}
		} else if (personnes == null || personnes.isEmpty()) {
			Assert.fail("Aucune personne n'a ete trouves dans votre liste");
		}
		log.debug("Sortie de la methode");
	}

	public void verifyDataPersonne(Personne person) {
		log.debug("Entree de la methode");
		if (person != null) {
			log.debug("idPersonne : " + person.getIdPersonne());
			Assert.assertNotNull(person.getIdPersonne());
			Assert.assertNotNull(person.getPrenom());
			Assert.assertNotNull(person.getNom());
			Assert.assertNotNull(person.getTaille());
			Assert.assertNotNull(person.getPoids());
			Assert.assertNotNull(person.getRue());
			Assert.assertNotNull(person.getCodePostal());
			Assert.assertNotNull(person.getVille());
			Assert.assertNotNull(person.getPays());
		} else if (person == null) {
			Assert.fail("Personne null");
		}
		log.debug("Sortie de la methode");
	}

	@Test
	public void testFindAllPersonnes() {
		log.debug("Entree de la methode");
		if (personnes != null) {
			log.debug("NB_PERSONNES_LIST: " + NB_PERSONNES_LIST + " , personnes.size(): " + personnes.size());
			Assert.assertEquals(NB_PERSONNES_LIST, personnes.size());
		}
		log.debug("Sortie de la methode");
	}

	@Test
	public void testFindByCriteria() {
		log.debug("Entree de la methode");
		List<Personne> personnesByPrenom = personneDao.findByPrenom(FIND_PERSONNES_BY_PRENOM);
		List<Personne> personnesByNom = personneDao.findByNom(FIND_PERSONNES_BY_NOM);
		List<Personne> personnesByCodePostal = personneDao.findByCodePostal(FIND_PERSONNES_BY_CODE_POSTAL);
		List<Personne> personnesByVille = personneDao.findByVille(FIND_PERSONNES_BY_VILLE);

		if (personnesByPrenom != null) {
			log.debug("NB_PERSONNES_FIND_BY_PRENOM: " + NB_PERSONNES_FIND_BY_PRENOM + " , personnesByPrenom.size(): "
					+ personnesByPrenom.size());
			Assert.assertEquals(NB_PERSONNES_FIND_BY_PRENOM, personnesByPrenom.size());
			verifyDataPersonnes(personnesByPrenom);
		} else {
			Assert.fail("Aucune personne avec le prenom '" + FIND_PERSONNES_BY_PRENOM
					+ "' n'a ete trouve dans votre base de données");
		}
		if (personnesByNom != null) {
			log.debug("NB_PERSONNES_FIND_BY_NOM: " + NB_PERSONNES_FIND_BY_NOM + " , personnesByNom.size(): "
					+ personnesByNom.size());
			Assert.assertEquals(NB_PERSONNES_FIND_BY_NOM, personnesByNom.size());
			verifyDataPersonnes(personnesByNom);
		} else {
			Assert.fail("Aucune personne avec le nom '" + FIND_PERSONNES_BY_NOM
					+ "' n'a ete trouve dans votre base de données");
		}
		if (personnesByCodePostal != null) {
			log.debug("NB_PERSONNES_FIND_BY_CODE_POSTAL: " + NB_PERSONNES_FIND_BY_CODE_POSTAL
					+ " , personnesByCodePostal.size(): " + personnesByCodePostal.size());
			Assert.assertEquals(NB_PERSONNES_FIND_BY_CODE_POSTAL, personnesByCodePostal.size());
			verifyDataPersonnes(personnesByCodePostal);
		} else {
			Assert.fail("Aucune personne avec le code postal '" + FIND_PERSONNES_BY_CODE_POSTAL
					+ "' n'a ete trouve dans votre base de données");
		}

		if (personnesByVille != null) {
			log.debug("NB_PERSONNES_FIND_BY_VILLE : " + NB_PERSONNES_FIND_BY_VILLE + " , personnesByVille.size(): "
					+ personnesByVille.size());
			Assert.assertEquals(NB_PERSONNES_FIND_BY_VILLE, personnesByVille.size());
			verifyDataPersonnes(personnesByVille);
		} else {
			Assert.fail("Aucune personne avec la ville '" + FIND_PERSONNES_BY_VILLE
					+ "' n'a ete trouve dans votre base de données");
		}
		log.debug("Sortie de la methode");
	}

	@Test
	public void testCreateUpdateDeletePersonne() {
		log.debug("Entree de la methode");
		Personne personCRUD = new Personne("Edouard", "Green", 100.0, 170.0, "rue du paradis", "76000", "Rouen",
				"France");
		personCRUD = personneDao.create(personCRUD);
		Assert.assertNotNull(personCRUD);
		Assert.assertNotNull(personCRUD.getIdPersonne());
		Assert.assertNotNull(personCRUD.getPrenom());
		Assert.assertNotNull(personCRUD.getNom());
		Assert.assertNotNull(personCRUD.getRue());
		Assert.assertNotNull(personCRUD.getVille());
		Assert.assertNotNull(personCRUD.getCodePostal());
		Assert.assertNotNull(personCRUD.getPays());
		log.debug("Created personCRUD : " + personCRUD);
		personCRUD = personneDao.findById(personCRUD.getIdPersonne());
		Assert.assertNotNull(personCRUD);
		personCRUD.setPrenom("Edouard Bis");
		personCRUD.setNom("Green Bis");
		personCRUD = personneDao.update(personCRUD);
		Assert.assertNotNull(personCRUD);
		personCRUD = personneDao.findById(personCRUD.getIdPersonne());
		log.debug("Updated personCRUD : " + personCRUD);
		Assert.assertEquals("Edouard Bis", personCRUD.getPrenom());
		Assert.assertEquals("Green Bis", personCRUD.getNom());
		Assert.assertTrue(personneDao.delete(personCRUD));
		personCRUD = personneDao.findById(personCRUD.getIdPersonne());
		Assert.assertNull(personCRUD);
		List<Personne> personnesFinal = personneDao.findAll();
		if (personnesFinal != null) {
			Assert.assertEquals(NB_PERSONNES_LIST, personnesFinal.size());
			log.debug(
					"personnesFinal.size() : " + personnesFinal.size() + " , NB_PERSONNES_LIST : " + NB_PERSONNES_LIST);
		} else if (personnesFinal == null || personnesFinal.isEmpty()) {
			Assert.fail("Aucune personne n'a ete trouve au final");
		}
		log.debug("Sortie de la methode");
	}

	@Test
	public void testCreateDeleteBatchPersonnes() {
		log.debug("Entree de la methode");
		List<Personne> personnesBatch = new ArrayList<Personne>();
		Personne personBatch1 = new Personne("Prenom-Batch-1", "Nom-Batch-1", 100.0, 170.0, "Rue-Batch-1",
				"Ville-Batch-1", "Code-Batch-1", "Pays--Batch-1");
		Personne personBatch2 = new Personne("Prenom-Batch-2", "Nom-Batch-2", 100.0, 170.0, "Rue-Batch-2",
				"Ville-Batch-2", "Code-Batch-2", "Pays--Batch-2");
		Personne personBatch3 = new Personne("Prenom-Batch-3", "Nom-Batch-3", 100.0, 170.0, "Rue-Batch-3",
				"Ville-Batch-3", "Code-Batch-3", "Pays--Batch-3");
		Personne personBatch4 = new Personne("Prenom-Batch-4", "Nom-Batch-4", 100.0, 170.0, "Rue-Batch-4",
				"Ville-Batch-4", "Code-Batch-4", "Pays--Batch-4");
		Personne personBatch5 = new Personne("Prenom-Batch-5", "Nom-Batch-5", 100.0, 170.0, "Rue-Batch-5",
				"Ville-Batch-5", "Code-Batch-5", "Pays--Batch-5");
		personnesBatch.add(personBatch1);
		personnesBatch.add(personBatch2);
		personnesBatch.add(personBatch3);
		personnesBatch.add(personBatch4);
		personnesBatch.add(personBatch5);
		Assert.assertNotNull(personneDao.createBatchPersonnes(personnesBatch));
		Assert.assertTrue(personneDao.deleteBatchPersonnes(personnesBatch));
		log.debug("Sortie de la methode");
	}

	@AfterClass
	public static void terminate() throws Exception {
		log.debug("Entree de la methode");
		serviceFacade = null;
		personneDao = null;
		personnes = null;
		if (context != null && context.isRunning()) {
			context.close();
		}
		log.debug("Sortie de la methode");
	}
}